/*
 * Copyright (c) 2014 - Zephyr Software LLC
 *
 * This file may be used and modified for non-commercial purposes as long as
 * all copyright, permission, and nonwarranty notices are preserved.
 * Redistribution is prohibited without prior written consent from Zephyr
 * Software.
 *
 * Please contact the authors for restrictions applying to commercial use.
 *
 * THIS SOURCE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Zephyr Software
 * e-mail: jwd@zephyr-software.com
 * URL   : http://www.zephyr-software.com/
 *
 */

#include <ostream>
#include <set>

namespace IRDB_SDK
{
	using namespace std;

	class BasicBlock_t;

	using BasicBlockSet_t = set<BasicBlock_t*>;

	class BasicBlock_t
	{
		protected:
			BasicBlock_t() {}
			BasicBlock_t(const BasicBlock_t& copy) = delete;
		public:

			bool                       getIsExitBlock()     const = 0;
			const InstructionVector_t& getInstructions()    const = 0;
			const BasicBlockSet_t&     getPredecessors()    const = 0;
			const BasicBlockSet_t&     getSuccessors()      const = 0;
			const BasicBlockSet_t&     getIndirectTargets() const = 0;
			const BasicBlock_t*        getFallthrough()     const = 0;
			const BasicBlock_t*        getTarget()          const = 0;


			bool           endsInBranch()            = 0;
			bool           endsInIndirectBranch()    = 0;
			bool           endsInConditionalBranch() = 0;
			Instruction_t* getBranchInstruction()    = 0;

			void           dump(ostream &os=std::cout) = 0 ;

	};
	ostream& operator<<(ostream& os, const BasicBlock_t& block);

}
