

namespace IRDB_SDK
{
	using namespace std;


	class CallGraphNode_t
	{
		protected:
			CallGraphNode_t() { }
			CallGraphNode_t(const CallGraphNode_t& copy) = delete;
		public:
			virtual ~CallGraphNode_t() { }

			bool isHellnode() const  =0;
			Function_t* getFunction() const  =0;
	};



	using CallGraphEdge_t    = pair<CallGraphNode_t*,CallGraphNode_t*>;
	using CallGraphNodeSet_t = set<CallGraphNode_t*>;
	using CallSite_t         = Instruction_t*;
	using CallSiteSet_t      = set<CallSite_t>;

	class Callgraph_t
	{
		protected:
			Callgraph_t() {}
			Callgraph_t(const Callgraph_t& copy) = delete;
		public:
			virtual ~Callgraph_t() { }
			virtual void addFile() = 0;

			virtual bool  edgeExists(const CallGraphEdge_t& edge) const = 0;

			virtual const CallGraphNodeSet_t& 
			              getCallersOfNode(CallGraphNode_t* const node)  const = 0;

			virtual const CallGraphNodeSet_t& 
			              getCalleesOfNode(CallGraphNode_t* const node)  const = 0;
			
			virtual const CallGraphNodeSet_t& ancestors, 
				       getAncestors
			               (
				        const Function_t* const fn, 
				        const bool skipHellNode = false
				       )                                            const = 0;
			virtual const  CallSiteSet_t& 
			               getCallSites(CallGraphNode_t* const n1)      const = 0;

			virtual void   dump(std::ostream& fout)                     const = 0;
			virtual string getNodeName(const CallGraphNode_t* const n1) const = 0;
			virtual string getCallsiteDisassembly(const CallSite_t &c)  const = 0;
			virtual bool   isReachable
			               (
			                CallGraphNode_t* const from, 
			                CallGraphNode_t* const to, 
			                bool skipHellNode = false
			               ) const = 0;

			virtual const  CallGraphNode_t& 
			               getDefaultHellNode()                         const = 0; 

			virtual CallGraphNode_t* 
			               findNode(Function_t* const fn)               const = 0;
	};

}
